#include "ArqWx.ListCtrl.h"


namespace ArqWx
{

//* ListCtrl(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style, const wxValidator& validator, const wxString& name)
//
// Constructor (identical to wxWidgets' native wxListCtrl)
//
// parameters:
// - parent (wxWindow*/NULL): parent window
// - id (wxWindowID/-1): window identifier
// - pos (const wxPoint&/wxDefaultPosition): object position
// - size (const wxSize&/wxDefaultSize): object size
// - style (long/wxLC_ICON): style
// - validator (const wxValidator&/wxDefaultValidator): validator
// - name (const wxString&/wxListCtrlNameStr): name

ListCtrl::ListCtrl(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style, const wxValidator& validator, const wxString& name)
: wxListCtrl(parent, id, pos, size, style, validator, name)
{
    this->rowBackgroundColoursAlternation = false;

    this->rowColours = new wxColour[2];
    this->rowColours[0].Set(255,255,255);
    this->rowColours[1].Set(245,245,245);
}

ListCtrl::~ListCtrl()
{
    delete[] this->rowColours;
}

//*** overriden methods

long ListCtrl::InsertItem(const wxListItem& info)
{
    long newItemIdx = wxListCtrl::InsertItem(info);
    this->RepaintRowBackgroundColours(info.m_itemId);
    return newItemIdx;
}

long ListCtrl::InsertItem(long index, const wxString& label)
{
    long newItemIdx = wxListCtrl::InsertItem(index, label);
    this->RepaintRowBackgroundColours(index);
    return newItemIdx;
}

long ListCtrl::InsertItem(long index, int imageIndex)
{
    long newItemIdx = wxListCtrl::InsertItem(index, imageIndex);
    this->RepaintRowBackgroundColours(index);
    return newItemIdx;
}

long ListCtrl::InsertItem(long index, const wxString& label, int imageIndex)
{
    long newItemIdx = wxListCtrl::InsertItem(index, label, imageIndex);
    this->RepaintRowBackgroundColours(index);
    return newItemIdx;
}

bool ListCtrl::DeleteItem(long item)
{
    bool rtn = wxListCtrl::DeleteItem(item);
    this->RepaintRowBackgroundColours(item);
    return rtn;
}

bool ListCtrl::SortItems(wxListCtrlCompare fnSortCallBack, long data)
{
    bool rtn = wxListCtrl::SortItems(fnSortCallBack, data);
    this->RepaintRowBackgroundColours();
    return rtn;
}


//***

//* void RepaintRowBackgroundColours()
//
// Repaints rows background colours with alternate colours
//
// parameters:
// - startItemIdx (long/0): 0-based index of start item from which redrawing has to be performed
// returns:
// - <none>

void ListCtrl::RepaintRowBackgroundColours(long startItemIdx)
{
    if ( this->rowBackgroundColoursAlternation ) { // if row background colours alternation is active
        for ( long itemIdx = startItemIdx ; itemIdx < this->GetItemCount() ; itemIdx++ ) { // for each item
            wxListItem listItem;

            listItem.m_itemId = itemIdx;
            this->GetItem(listItem); // gets item

            listItem.SetBackgroundColour(this->rowColours[itemIdx % 2]); // sets background colour (different colours for even and odd rows)
            this->SetItem(listItem); // sets item
        }
    }
}

//* void RepaintRowBackgroundDefaultColours()
//
// Repaints rows background with default colours
//
// parameters:
// - <none>
// returns:
// - <none>

void ListCtrl::RepaintRowBackgroundDefaultColours()
{
    for ( long itemIdx = 0 ; itemIdx < this->GetItemCount() ; itemIdx++ ) { // for each item
        wxListItem listItem;

        listItem.m_itemId = itemIdx;
        this->GetItem(listItem); // gets item

        listItem.SetBackgroundColour(wxColour(255,255,255)); // sets background colour
        this->SetItem(listItem); // sets item
    }
}

//* bool GetRowBackgroundColourAlternation() const
//
// Gets row background colours alternation state
//
// parameters:
// - <none>
// returns:
// - (bool): Row background colours alternation state

bool ListCtrl::GetRowBackgroundColourAlternation() const
{
    return this->rowBackgroundColoursAlternation;
}

//* void SetRowBackgroundColourAlternation(bool rowBackgroundColoursAlternation)
//
// Sets and applies row background colours alternation state
//
// parameters:
// - rowBackgroundColoursAlternation (bool): row background colours alternation state
// returns:
// - <none>

void ListCtrl::SetRowBackgroundColourAlternation(bool rowBackgroundColoursAlternation)
{
    this->rowBackgroundColoursAlternation = rowBackgroundColoursAlternation;

    if ( this->rowBackgroundColoursAlternation ) {
        this->RepaintRowBackgroundColours();
    }
    else {
        this->RepaintRowBackgroundDefaultColours();
    }
}


//* void SetRowBackgroundColours(wxColour evenColour, wxColour oddColour)
//
// Changes row colours
//
// parameters:
// - evenColour (wxColour): colour for even rows
// - oddColour (wxColour): colour for odd rows
// returns:
// - <none>

void ListCtrl::SetRowBackgroundColours(wxColour evenColour, wxColour oddColour)
{
    this->rowColours[0] = evenColour;
    this->rowColours[1] = oddColour;

    this->RepaintRowBackgroundColours();
}

//* long GetIdenticalItemsCount(const wxString& itemValue)
//
// Gets number of items with identical value
//
// parameters:
// - itemValue (const wxString&): item value to looked for
// returns:
// - (long): item count with identical value

long ListCtrl::GetIdenticalItemsCount(const wxString& itemValue)
{
    long itemValueCount = 0;
    long pos = -1;

    while ( (pos = this->FindItem(pos+1, itemValue)) != -1 ) // while given item can be found into given listcontrol
        itemValueCount++; // counts it

    return itemValueCount;
}


//* long AddColumn(const wxListItem& info)
//
// Adds a new column
//
// parameters:
// - info (const wxListItem&): list item of column to added
// returns:
// - (long): 0-based index of new column

long ListCtrl::AddColumn(const wxListItem& info)
{
    return this->InsertColumn(this->GetColumnCount(), info);
}

//* long AddColumn(const wxString& heading, int format, int width)
//
// Adds a new column
//
// parameters:
// - heading (const wxString&): header of column
// - format (int/wxLIST_FORMAT_LEFT): format of header column
// - width (int/-1): width of column
// returns:
// - (long): 0-based index of new column

long ListCtrl::AddColumn(const wxString& heading, int format, int width)
{
    return this->InsertColumn(this->GetColumnCount(), heading, format, width);
}


//* void ClearColumn(int colIdx)
//
// Clears a whole column
//
// parameters:
// - colIdx (int): 0-based index of column
// returns:
// - <none>

void ListCtrl::ClearColumn(int colIdx)
{
    for ( long itemIdx = 0 ; itemIdx < this->GetItemCount() ; itemIdx++ ) {
        this->SetItem(itemIdx, colIdx, wxEmptyString); // clears element
    }
}


//* long GetSelectedItemIndex() const
//
// Gets selected item index
//
// parameters:
// - <none>
// returns:
// - (long): selected item index, -1 if none is selected

long ListCtrl::GetSelectedItemIndex() const
{
    return this->GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
}


//*** Get_() methods

//* wxString GetString(long itemIdx, int colIdx) const
//
// Gets value of an element (element = {row ; col}) of type 'string'
//
// parameters:
// - itemIdx (long): 0-based index of row
// - colIdx (int/0): 0-based index of column
// returns:
// - (wxString): element value as string

wxString ListCtrl::GetString(long itemIdx, int colIdx) const
{
    wxListItem listItem;

    listItem.m_itemId = itemIdx; // sets row
    listItem.m_col = colIdx; // sets column
    listItem.m_mask = wxLIST_MASK_TEXT; // parses item text
    this->GetItem(listItem); // gets item

    return listItem.m_text; // returns value as string
}

//* bool GetBoolean(long itemIdx, int colIdx) const
//
// Gets value of an element (element = {row ; col}) of type 'boolean'
//
// parameters:
// - itemIdx (long): 0-based index of row
// - colIdx (int/0): 0-based index of column
// returns:
// - (bool): element value as boolean

bool ListCtrl::GetBoolean(long itemIdx, int colIdx) const
{
    return this->GetString(itemIdx, colIdx).IsSameAs("true");
}

//* long GetLong(long itemIdx, int colIdx) const
//
// Gets value of an element (element = {row ; col}) of type 'integer'
//
// parameters:
// - itemIdx (long): 0-based index of row
// - colIdx (int/0): 0-based index of column
// returns:
// - (long): element value as integer

long ListCtrl::GetLong(long itemIdx, int colIdx) const
{
    long i;
    this->GetString(itemIdx, colIdx).ToLong(&i);
    return i;
}

//* double GetDouble(long itemIdx, int colIdx) const
//
// Gets value of an element (element = {row ; col}) of type 'double'
//
// parameters:
// - itemIdx (long): 0-based index of row
// - colIdx (int/0): 0-based index of column
// returns:
// - (double): element value as double

double ListCtrl::GetDouble(long itemIdx, int colIdx) const
{
    double d;
    this->GetString(itemIdx, colIdx).ToDouble(&d);
    return d;
}

//* time_t GetDateTime(long itemIdx, int colIdx) const
//
// Gets value of an element (element = {row ; col}) of type 'time_t'
//
// parameters:
// - itemIdx (long): 0-based index of row
// - colIdx (int/0): 0-based index of column
// returns:
// - (time_t): element value as time_t (unsigned integer)

time_t ListCtrl::GetDateTime(long itemIdx, int colIdx) const
{
    wxDateTime dt;
    dt.ParseDateTime(this->GetString(itemIdx, colIdx));
    return dt.GetTicks();
}


//* void GetItemRow(long itemIdx, wxString elements[]) const
//
// Gets all item elements and stores them into given string array
//
// parameters:
// - itemIdx (long): 0-based index of row
// - elements (wxString[]): elements of the whole item
// returns:
// - <none>

void ListCtrl::GetItemRow(long itemIdx, wxString elements[]) const
{
    for ( int colIdx = 0 ; colIdx < this->GetColumnCount() ; colIdx++ ) { // for each element
        elements[colIdx] = this->GetString(itemIdx, colIdx);
    }
}

//* void GetItemRow(long itemIdx, wxArrayString& values) const
//
// Gets all item elements and stores them into given string array object
//
// parameters:
// - itemIdx (long): 0-based index of row
// - elements (wxArrayString&): elements of the whole item
// returns:
// - <none>

void ListCtrl::GetItemRow(long itemIdx, wxArrayString& values) const
{
    values.Empty();
    values.Alloc(this->GetColumnCount());

    for ( int colIdx = 0 ; colIdx < this->GetColumnCount() ; colIdx++ ) { // for each element
        values.Add(this->GetString(itemIdx, colIdx));
    }
}


//*** Set_() methods

//* void SetString(long itemIdx, int colIdx, const wxString& str)
//
// Sets value of an element (element = {row ; col}) of type 'string'
//
// parameters:
// - itemIdx (long): 0-based index of row
// - colIdx (int): 0-based index of column
// - str (const wxString&): string value
// returns:
// - <none>

void ListCtrl::SetString(long itemIdx, int colIdx, const wxString& str)
{
    this->SetItem(itemIdx, colIdx, str);
}

//* void SetBoolean(long itemIdx, int colIdx, bool bval)
//
// Sets value of an element (element = {row ; col}) of type 'boolean'
//
// parameters:
// - itemIdx (long): 0-based index of row
// - colIdx (int): 0-based index of column
// - bval (bool): boolean value
// returns:
// - <none>

void ListCtrl::SetBoolean(long itemIdx, int colIdx, bool bval)
{
    if ( bval ) {
        this->SetItem(itemIdx, colIdx, "true");
    }
    else {
        this->SetItem(itemIdx, colIdx, "false");
    }
}

//* void SetLong(long itemIdx, int colIdx, long ival)
//
// Sets value of an element (element = {row ; col}) of type 'integer'
//
// parameters:
// - itemIdx (long): 0-based index of row
// - colIdx (int): 0-based index of column
// - ival (long): integer value
// returns:
// - <none>

void ListCtrl::SetLong(long itemIdx, int colIdx, long ival)
{
    this->SetItem(itemIdx, colIdx, wxString::Format("%ld", ival));
}

//* void SetDouble(long itemIdx, int colIdx, double dval)
//
// Sets value of an element (element = {row ; col}) of type 'double'
//
// parameters:
// - itemIdx (long): 0-based index of row
// - colIdx (int): 0-based index of column
// - dval (double): double value
// returns:
// - <none>

void ListCtrl::SetDouble(long itemIdx, int colIdx, double dval)
{
    this->SetItem(itemIdx, colIdx, wxString::Format("%lf", dval));
}

//* void SetDateTime(long itemIdx, int colIdx, time_t tval)
//
// Sets value of an element (element = {row ; col}) of type 'time_t'
//
// parameters:
// - itemIdx (long): 0-based index of row
// - colIdx (int): 0-based index of column
// - tval (time_t): time_t value
// returns:
// - <none>

void ListCtrl::SetDateTime(long itemIdx, int colIdx, time_t tval)
{
    this->SetItem(itemIdx, colIdx, wxDateTime(tval).FormatISODate() + " " + wxDateTime(tval).FormatISOTime());
}


//* long AddItemRow(const wxString elements[])
//
// Adds the whole item using given elements stored in string array
//
// parameters:
// - elements (const wxString[]): elements to use to initialize item
// returns:
// - (long): 0-based index of new item

long ListCtrl::AddItemRow(const wxString elements[])
{
    int itemIdx = this->GetItemCount(); // gets items count
    this->InsertItem(itemIdx, wxEmptyString); // inserts a new item

    return this->SetItemRow(itemIdx, elements);
}

//* long AddItemRow(const wxArrayString& elements)
//
// Adds the whole item using given elements stored in string array object
//
// parameters:
// - elements (const wxArrayString&): elements to use to initialize item
// returns:
// - (long): 0-based index of new item

long ListCtrl::AddItemRow(const wxArrayString& elements)
{
    int itemIdx = this->GetItemCount(); // gets items count
    this->InsertItem(itemIdx, wxEmptyString); // inserts a new item

    return this->SetItemRow(itemIdx, elements);
}


//* long SetItemRow(long itemIdx, const wxString elements[], bool autoInsertion)
//
// Sets the whole item using given elements stored in string array
//
// parameters:
// - itemIdx (long): 0-based index of item to set
// - elements (const wxString[]): elements to use to initialize item
// - autoInsertion (bool/false): enables automatic insertion of item if necessary
// returns:
// - (long): 0-based index of item

long ListCtrl::SetItemRow(long itemIdx, const wxString elements[], bool autoInsertion)
{
    // creation of an item if necessary
    if ( autoInsertion ) {
        for ( int emptyItem = this->GetItemCount() ; emptyItem <= itemIdx ; emptyItem++ ) // for each non-existing item
            this->InsertItem(emptyItem, wxEmptyString);
    }

    // setting of the whole row
    for ( int colIdx = 0 ; colIdx < this->GetColumnCount() ; colIdx++ ) { // for each element
        this->SetItem(itemIdx, colIdx, elements[colIdx]);
    }

    return itemIdx;
}

//* long SetItemRow(long itemIdx, const wxArrayString& elements, bool autoInsertion)
//
// Sets the whole item using given elements stored in string array objet
//
// parameters:
// - itemIdx (long): 0-based index of item to set
// - elements (const wxArrayString&): elements to use to initialize item
// - autoInsertion (bool/false): enables automatic insertion of item if necessary
// returns:
// - (long): 0-based index of item

long ListCtrl::SetItemRow(long itemIdx, const wxArrayString& elements, bool autoInsertion)
{
    // creation of an item if necessary
    if ( autoInsertion ) {
        for ( int emptyItemIdx = this->GetItemCount() ; emptyItemIdx <= itemIdx ; emptyItemIdx++ )
            this->InsertItem(emptyItemIdx, wxEmptyString);
    }

    // setting of the whole row
    // unmatching counts for both listcontrol columns count and array string are set to empty string by default
    for ( int colIdx = 0 ; (colIdx < this->GetColumnCount()) && (colIdx < elements.GetCount()) ; colIdx++ ) { // for each element existing in both columns list and array string
        this->SetItem(itemIdx, colIdx, elements[colIdx]);
    }

    return itemIdx;
}

//* void MoveItem(long itemIdx, long newItemIdx)
//
// Moves an item to another index
//
// parameters:
// - itemIdx (long): 0-based index of item to move
// - newItemIdx (long): 0-based index of new position of item
// returns:
// <none>

void ListCtrl::MoveItem(long itemIdx, long newItemIdx)
{
    wxArrayString itemRow;

    this->GetItemRow(itemIdx, itemRow); // gets current item values
    this->DeleteItem(itemIdx); // removes item
    this->InsertItem(newItemIdx, ""); // creates new item
    this->SetItemRow(newItemIdx, itemRow, true); // restores item values
}

} // ArqWx namespace end
