#include <wx/wx.h>
#include "ArqWx.RegEx.h"


namespace ArqWx
{

//* constructors, accessors


//* RegEx(const wxString& pattern, const wxString& text, int flags)
//
// Builds a new RegEx object with given pattern and flags to apply to given text
//
// parameters:
// - pattern (const wxString&): regex pattern to use
// - text (const wxString&): text to apply regex to
// - flags (int/wxRE_DEFAULT): flags

RegEx::RegEx(const wxString& pattern, const wxString& text, int flags)
{
    this->InitRegEx(pattern, text, flags);
}


//* RegEx(const wxString& pattern, int flags)
//
// Builds a new RegEx object with given pattern and flags
//
// parameters:
// - pattern (const wxString&): regex pattern to use
// - flags (int/wxRE_DEFAULT): flags

RegEx::RegEx(const wxString& pattern, int flags)
{
    this->InitRegEx(pattern, wxEmptyString, flags);
}


bool RegEx::InitRegEx(const wxString& pattern, const wxString& text, int flags)
{
    this->pattern = pattern;
    this->text = text;
    this->flags = flags;

    this->matchFlags = 0;

    this->workingPattern = this->pattern; // creates a working pattern; as pattern will be modified then original pattern won't be altered
    this->ExtractGroups(); // creates groups list and modifies working pattern

    return wxRegEx::Compile(this->workingPattern, flags); // compiles inherited regex using working pattern
}


//* void SetText(const wxString& text)
//
// Sets text to apply regex to
//
// parameters:
// - text (const wxString&): text to apply regex to
// returns:
// - <none>

void RegEx::SetText(const wxString& text)
{
    this->text = text;
}


//*** named groups methods

//* void ExtractGroups()
//
// Extracts groups name from original pattern (scheme is '(?<groupname>...)' where '...' is regex part referring to group
// Updates original pattern by removing groups name so that pattern could be compiled using wxRegEx methods
//
// parameters:
// - <none>
// returns:
// - <none>

void RegEx::ExtractGroups()
{
    // [^0-9][0-9a-zA-Z]+ : 1 or more alphanumeric characters (first one cannot be a numeric)...
    // ([^0-9][0-9a-zA-Z]+) : ... grouped...
    // <([^0-9][0-9a-zA-Z]+)> : ... between < > or ' '...
    // \\?<([^0-9][0-9a-zA-Z]+)> : ... preceded by litteral '?'...
    // \\?<([^0-9][0-9a-zA-Z]+)>([^)]+) : ... followed by 1 or more characters (any character except ')')...
    // \\(\\?<([^0-9][0-9a-zA-Z]+)>[^)]+\\) : ... between litteral ( )
    wxRegEx groupsRegEx("\\(\\?<([^0-9][0-9a-zA-Z]+)>([^)]+)\\)", wxRE_ADVANCED);
    wxString pattern2 = this->workingPattern; // copy of working pattern

    while ( groupsRegEx.Matches(pattern2) ) { // while a named group can be found into pattern...
        size_t start, length;
        groupsRegEx.GetMatch(&start, &length, 1); // sets start and length of match
        this->groupsList.Add(pattern2.Mid(start, length)); // extracts text for group name and stores it
        pattern2.Remove(0, start+length); // removes used part of the pattern, from start to already matched part
    }

    groupsRegEx.Replace(&this->workingPattern, "(\\2)"); // removes all named groups from working pattern
}


//* wxString Groups(const wxString& text, const wxString& groupname)
//
// Extracts matched part of given text for given group name
//
// parameters:
// - text (const wxString&): text to apply regex to
// - groupname (const wxString&): group name to extract
// returns:
// - (wxString): matched part of text

wxString RegEx::Groups(const wxString& text, const wxString& groupname)
{
    int index; // index of group name
    wxString matchedText = wxEmptyString; // return string

    index = this->groupsList.Index(groupname); // gets index from given group name into groups list
    if ( index != wxNOT_FOUND ) // if group name can be found
        if ( this->Matches(text, this->matchFlags) ) // if regex matches text
            matchedText = this->GetMatch(text, index+1); // sets matched part as return string

    return matchedText; // returns matched part of text
}

//* wxString Groups(const wxString& groupname)
//
// Extracts matched part of predefined text for given group name
//
// parameters:
// - groupname (const wxString&): group name to extract
// returns:
// - (wxString): matched part of text

wxString RegEx::Groups(const wxString& groupname)
{
    return this->Groups(this->text, groupname); // returns matched part of text
}


//* wxString Groups(const wxString& text, size_t index)
//
// Extracts matched part of given text for given group index
//
// parameters:
// - text (const wxString&): text to apply regex to
// - index (size_t): group index to extract
// returns:
// - (wxString): matched part of text

wxString RegEx::Groups(const wxString& text, size_t index)
{
    wxString matchedText = wxEmptyString; // matched part of text

    if ( this->Matches(text, this->matchFlags) ) // if regex matches text
        matchedText = this->GetMatch(text, index); // sets matched part as return string

    return matchedText; // returns matched part of text
}

//* wxString Groups(size_t index)
//
// Extracts matched part of predefined text for given group index
//
// parameters:
// - index (size_t): group index to extract
// returns:
// - (wxString): matched part of text

wxString RegEx::Groups(size_t index)
{
    return this->Groups(this->text, index); // returns matched part of text
}


//*** whole text match' methods

//* size_t GetTotalMatchCount(const wxString& text) const
//
// Gets total match count when regex is applied to given text
//
// parameters:
// - text (const wxString&): text to apply regex to
// returns:
// - (size_t): total match count

size_t RegEx::GetTotalMatchCount(const wxString& text) const
{
    wxString text2 = text; // copy of expression
    wxRegEx workingRegEx(this->workingPattern); // working regex
    size_t totalMatchCount = 0; // total match count

    while ( workingRegEx.Matches(text2, this->matchFlags) ) { // while a group can be found into expression...
        size_t start, length;
        size_t count = this->GetMatchCount() - 1; // initializes current match' group count
        totalMatchCount += count; // adds current match' group count to total count
        workingRegEx.GetMatch(&start, &length, count); // sets start and length of last match
        text2.Remove(0, start+length); // removes used part of text, from start to last character of last match
    }

    return totalMatchCount; // returns match count
}

//* size_t GetTotalMatchCount() const
//
// Gets total match count when regex is applied to predefined text
//
// parameters:
// - <none>
// returns:
// - (size_t): total match count

size_t RegEx::GetTotalMatchCount() const
{
    return this->GetTotalMatchCount(this->text); // returns match count
}


//* wxString GetTotalMatch(const wxString& text, size_t index) const
//
// Extracts matched part of given text for given group index
//
// parameters:
// - text (const wxString&): text to apply regex to
// - index (size_t): group index to extract
// returns:
// - (wxString): matched part of text

wxString RegEx::GetTotalMatch(const wxString& text, size_t index) const
{
    size_t relativeIndex = index; // relative index
    wxRegEx workingRegEx(this->workingPattern); // working regex
    wxString text2 = text; // copy of expression
    bool matchFound = false; // match found flag
    wxString matchedText = wxEmptyString; // matched part of text

    while ( workingRegEx.Matches(text2, this->matchFlags) && !matchFound ) { // while a group can be found into expression...
        size_t start, length;
        size_t count = this->GetMatchCount() - 1; // initializes current match' group count
        if ( (int)(relativeIndex - count) <= 0 ) { // if relative index is below count (/index can be found into current match)
            matchedText = workingRegEx.GetMatch(text2, relativeIndex); // gets matched text
            matchFound = true; // sets match found as true
        }
        else { // if relative index is greater than count (index cannot be found into current match)
            relativeIndex -= count; // updates relative index (/go to next match)
            workingRegEx.GetMatch(&start, &length, count); // sets start and length of last match
            text2.Remove(0, start+length); // removes used part of text, from start to last character of last match
        }
    }

    return matchedText; // returns matched part of text
}

//* wxString GetTotalMatch(size_t index) const
//
// Extracts matched part of predefined text for given group index
//
// parameters:
// - index (size_t): group index to extract
// returns:
// - (wxString): matched part of text

wxString RegEx::GetTotalMatch(size_t index) const
{
    return this->GetTotalMatch(this->text, index); // returns matched part of text
}


//*** overridden methods

bool RegEx::Compile(const wxString& pattern, int flags)
{
    return this->InitRegEx(pattern, wxEmptyString, flags);
}


bool RegEx::Matches(const wxChar* text, int flags)
{
    this->text = text;
    this->matchFlags = flags;
    return wxRegEx::Matches(text, this->matchFlags);
}

bool RegEx::Matches(const wxChar* text, int flags, size_t len)
{
    this->text = text;
    this->matchFlags = flags;
    return wxRegEx::Matches(text, this->matchFlags, len);
}

bool RegEx::Matches(const wxString& text, int flags)
{
    this->text = text;
    this->matchFlags = flags;
    return wxRegEx::Matches(text, this->matchFlags);
}

} // ArqWx namespace end
